@@ -15,7 +15,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
     
-    $Id: tap_eq.c,v 1.16 2004/01/24 20:55:13 root Exp $
+    $Id: tap_eq.c,v 1.7 2006/08/09 12:03:24 tszilagyi Exp $
 */
 
 
@@ -127,32 +127,21 @@ static
 void
 activate_eq(LADSPA_Handle instance) {
 
-	eq *ptr = (eq *)instance;
-	biquad *filters = ptr->filters;
-	float fs = ptr->fs;
+        eq *ptr = (eq *)instance;
+        biquad *filters = ptr->filters;
 
 	biquad_init(&filters[0]);
-	eq_set_params(&filters[0], 100.0f, 0.0f, BWIDTH, fs);
 	biquad_init(&filters[1]);
-	eq_set_params(&filters[1], 200.0f, 0.0f, BWIDTH, fs);
 	biquad_init(&filters[2]);
-	eq_set_params(&filters[2], 400.0f, 0.0f, BWIDTH, fs);
 	biquad_init(&filters[3]);
-	eq_set_params(&filters[3], 1000.0f, 0.0f, BWIDTH, fs);
 	biquad_init(&filters[4]);
-	eq_set_params(&filters[4], 3000.0f, 0.0f, BWIDTH, fs);
 	biquad_init(&filters[5]);
-	eq_set_params(&filters[5], 6000.0f, 0.0f, BWIDTH, fs);
 	biquad_init(&filters[6]);
-	eq_set_params(&filters[6], 12000.0f, 0.0f, BWIDTH, fs);
 	biquad_init(&filters[7]);
-	eq_set_params(&filters[7], 15000.0f, 0.0f, BWIDTH, fs);
-
-	ptr->filters = filters;
-	ptr->fs = fs;
 }
 
 
+
 static
 void
 cleanup_eq(LADSPA_Handle instance) {
@@ -235,6 +224,8 @@ instantiate_eq(const LADSPA_Descriptor *descriptor, unsigned long s_rate) {
 	float fs;
 
 	fs = s_rate;
+
+	memset(ptr, 0, sizeof(eq));
 	
 	filters = calloc(8, sizeof(biquad));
 
@@ -242,6 +233,39 @@ instantiate_eq(const LADSPA_Descriptor *descriptor, unsigned long s_rate) {
 	ptr->fs = fs;
 	ptr->run_adding_gain = 1.0f;
 
+	ptr->old_ch0f = 100.0f;
+	ptr->old_ch0g = 0;
+
+	ptr->old_ch1f = 200.0f;
+	ptr->old_ch1g = 0;
+
+	ptr->old_ch2f = 400.0f;
+	ptr->old_ch2g = 0;
+
+	ptr->old_ch3f = 1000.0f;
+	ptr->old_ch3g = 0;
+
+	ptr->old_ch4f = 3000.0f;
+	ptr->old_ch4g = 0;
+
+	ptr->old_ch5f = 6000.0f;
+	ptr->old_ch5g = 0;
+
+	ptr->old_ch6f = 12000.0f;
+	ptr->old_ch6g = 0;
+
+	ptr->old_ch7f = 15000.0f;
+	ptr->old_ch7g = 0;
+
+	eq_set_params(&filters[0], 100.0f, 0.0f, BWIDTH, fs);
+	eq_set_params(&filters[1], 200.0f, 0.0f, BWIDTH, fs);
+	eq_set_params(&filters[2], 400.0f, 0.0f, BWIDTH, fs);
+	eq_set_params(&filters[3], 1000.0f, 0.0f, BWIDTH, fs);
+	eq_set_params(&filters[4], 3000.0f, 0.0f, BWIDTH, fs);
+	eq_set_params(&filters[5], 6000.0f, 0.0f, BWIDTH, fs);
+	eq_set_params(&filters[6], 12000.0f, 0.0f, BWIDTH, fs);
+	eq_set_params(&filters[7], 15000.0f, 0.0f, BWIDTH, fs);
+
 	return (LADSPA_Handle)ptr;
 }
 
@@ -252,22 +276,22 @@ run_eq(LADSPA_Handle instance, unsigned long sample_count) {
 
 	eq * ptr = (eq *)instance;
 
-	const LADSPA_Data ch0f = *(ptr->ch0f);
-	const LADSPA_Data ch0g = *(ptr->ch0g);
-	const LADSPA_Data ch1f = *(ptr->ch1f);
-	const LADSPA_Data ch1g = *(ptr->ch1g);
-	const LADSPA_Data ch2f = *(ptr->ch2f);
-	const LADSPA_Data ch2g = *(ptr->ch2g);
-	const LADSPA_Data ch3f = *(ptr->ch3f);
-	const LADSPA_Data ch3g = *(ptr->ch3g);
-	const LADSPA_Data ch4f = *(ptr->ch4f);
-	const LADSPA_Data ch4g = *(ptr->ch4g);
-	const LADSPA_Data ch5f = *(ptr->ch5f);
-	const LADSPA_Data ch5g = *(ptr->ch5g);
-	const LADSPA_Data ch6f = *(ptr->ch6f);
-	const LADSPA_Data ch6g = *(ptr->ch6g);
-	const LADSPA_Data ch7f = *(ptr->ch7f);
-	const LADSPA_Data ch7g = *(ptr->ch7g);
+	const LADSPA_Data ch0f = LIMIT(*(ptr->ch0f),40.0f,280.0f);
+	const LADSPA_Data ch0g = LIMIT(*(ptr->ch0g),-50.0f,20.0f);
+	const LADSPA_Data ch1f = LIMIT(*(ptr->ch1f),100.0f,500.0f);
+	const LADSPA_Data ch1g = LIMIT(*(ptr->ch1g),-50.0f,20.0f);
+	const LADSPA_Data ch2f = LIMIT(*(ptr->ch2f),200.0f,1000.0f);
+	const LADSPA_Data ch2g = LIMIT(*(ptr->ch2g),-50.0f,20.0f);
+	const LADSPA_Data ch3f = LIMIT(*(ptr->ch3f),400.0f,2800.0f);
+	const LADSPA_Data ch3g = LIMIT(*(ptr->ch3g),-50.0f,20.0f);
+	const LADSPA_Data ch4f = LIMIT(*(ptr->ch4f),1000.0f,5000.0f);
+	const LADSPA_Data ch4g = LIMIT(*(ptr->ch4g),-50.0f,20.0f);
+	const LADSPA_Data ch5f = LIMIT(*(ptr->ch5f),3000.0f,9000.0f);
+	const LADSPA_Data ch5g = LIMIT(*(ptr->ch5g),-50.0f,20.0f);
+	const LADSPA_Data ch6f = LIMIT(*(ptr->ch6f),6000.0f,18000.0f);
+	const LADSPA_Data ch6g = LIMIT(*(ptr->ch6g),-50.0f,20.0f);
+	const LADSPA_Data ch7f = LIMIT(*(ptr->ch7f),10000.0f,20000.0f);
+	const LADSPA_Data ch7g = LIMIT(*(ptr->ch7g),-50.0f,20.0f);
 
 	const LADSPA_Data * input = ptr->input;
 	LADSPA_Data * output = ptr->output;
@@ -368,22 +392,22 @@ run_adding_eq(LADSPA_Handle instance, unsigned long sample_count) {
 
 	eq * ptr = (eq *)instance;
 
-	const LADSPA_Data ch0f = *(ptr->ch0f);
-	const LADSPA_Data ch0g = *(ptr->ch0g);
-	const LADSPA_Data ch1f = *(ptr->ch1f);
-	const LADSPA_Data ch1g = *(ptr->ch1g);
-	const LADSPA_Data ch2f = *(ptr->ch2f);
-	const LADSPA_Data ch2g = *(ptr->ch2g);
-	const LADSPA_Data ch3f = *(ptr->ch3f);
-	const LADSPA_Data ch3g = *(ptr->ch3g);
-	const LADSPA_Data ch4f = *(ptr->ch4f);
-	const LADSPA_Data ch4g = *(ptr->ch4g);
-	const LADSPA_Data ch5f = *(ptr->ch5f);
-	const LADSPA_Data ch5g = *(ptr->ch5g);
-	const LADSPA_Data ch6f = *(ptr->ch6f);
-	const LADSPA_Data ch6g = *(ptr->ch6g);
-	const LADSPA_Data ch7f = *(ptr->ch7f);
-	const LADSPA_Data ch7g = *(ptr->ch7g);
+	const LADSPA_Data ch0f = LIMIT(*(ptr->ch0f),40.0f,280.0f);
+	const LADSPA_Data ch0g = LIMIT(*(ptr->ch0g),-50.0f,20.0f);
+	const LADSPA_Data ch1f = LIMIT(*(ptr->ch1f),100.0f,500.0f);
+	const LADSPA_Data ch1g = LIMIT(*(ptr->ch1g),-50.0f,20.0f);
+	const LADSPA_Data ch2f = LIMIT(*(ptr->ch2f),200.0f,1000.0f);
+	const LADSPA_Data ch2g = LIMIT(*(ptr->ch2g),-50.0f,20.0f);
+	const LADSPA_Data ch3f = LIMIT(*(ptr->ch3f),400.0f,2800.0f);
+	const LADSPA_Data ch3g = LIMIT(*(ptr->ch3g),-50.0f,20.0f);
+	const LADSPA_Data ch4f = LIMIT(*(ptr->ch4f),1000.0f,5000.0f);
+	const LADSPA_Data ch4g = LIMIT(*(ptr->ch4g),-50.0f,20.0f);
+	const LADSPA_Data ch5f = LIMIT(*(ptr->ch5f),3000.0f,9000.0f);
+	const LADSPA_Data ch5g = LIMIT(*(ptr->ch5g),-50.0f,20.0f);
+	const LADSPA_Data ch6f = LIMIT(*(ptr->ch6f),6000.0f,18000.0f);
+	const LADSPA_Data ch6g = LIMIT(*(ptr->ch6g),-50.0f,20.0f);
+	const LADSPA_Data ch7f = LIMIT(*(ptr->ch7f),10000.0f,20000.0f);
+	const LADSPA_Data ch7g = LIMIT(*(ptr->ch7g),-50.0f,20.0f);
 
 	const LADSPA_Data * input = ptr->input;
 	LADSPA_Data * output = ptr->output;
